#include <iostream>
#include <cstdlib>

#define N_BLOCKS			16  // number of blocks
#define N_THREADS_PER_BLOCK	32  // number of threads per block
#define N_RN				(100 * N_BLOCKS * N_THREADS_PER_BLOCK)

#define ITERATIONS			10000 // number of runs

unsigned Tausworthe88(unsigned &z1, unsigned &z2, unsigned &z3)
{
	unsigned b;

	// Three-step generator with period 2^88
	b = (((z1 << 13) ^ z1) >> 19);
	z1 = (((z1 & 4294967294) << 12) ^ b);

	b = (((z2 << 2)  ^ z2) >> 25);
	z2 = (((z2 & 4294967288) << 4)  ^ b);

	b = (((z3 << 3)  ^ z3) >> 11);
	z3 = (((z3 & 4294967280) << 17) ^ b);

	return z1 ^ z2 ^ z3;
}

unsigned LCRNG(unsigned &z)  
{  
	const unsigned a = 1664525;
	const unsigned c = 1013904223;
	return z = a * z + c;
}

float TauswortheLCRNG(unsigned &z1, unsigned &z2, unsigned &z3, unsigned &z)
{
	// combine both generators and normalize 0...2^32 to 0...1
	return (Tausworthe88(z1, z2, z3) ^ LCRNG(z)) * 2.3283064365e-10;
}


int main (int argc, char const* argv[])
{
	// create 4 seed values:
	unsigned z1 = static_cast<unsigned>(4294967295 * drand48());	// Tausworthe seeds
	unsigned z2 = static_cast<unsigned>(4294967295 * drand48());
	unsigned z3 = static_cast<unsigned>(4294967295 * drand48());

	unsigned z  = static_cast<unsigned>(4294967295 * drand48());	// LCRNG seed

	float randoms[N_RN];

	for(unsigned i=0; i<ITERATIONS; i++)
	{
		for(unsigned i=0; i<N_RN; i++)
			randoms[i] = TauswortheLCRNG(z1, z2, z3, z);
	}

	return 0;
}