#include <iostream>
#include <cstdlib>
#include <cmath>

#define L 64 	// lattice length; must be even number!
#define D 2 	// dimensions
#define N 4096	// number of spins on square lattice (N=L^D)
#define Q 2 	// number of Potts states

#define N_SWEEPS	50000


int main(int argc, char const *argv[])
{
	// each spin has value 0,..,Q-1
	unsigned short spins[N];


	// calculate lattice volume elements:
	unsigned volume[D];
	for(unsigned i=0; i<=D; i++)
	{
		if(i == 0)
			volume[i] = 1;
		else
			volume[i] = volume[i-1] * L;
	}	


	// initialise lattice with random spin states:

	for(unsigned i=0; i<N; i++)
		spins[i] = floor(drand48() * Q);


	// neighborhood table:
	unsigned neighbors[2*D*N];


	// calculate neighborhood table:
	for(unsigned i=0; i<N; i++)
	{
		unsigned short c=0;

		for(unsigned short dim=0; dim<D; dim++)	// dimension loop
		{
			for(short dir=-1; dir<=1; dir+=2)	// two directions in each dimension
			{
				// neighbor's id in spin list:
				int npos = i + dir * volume[dim];

				// periodic boundary conditions:
				int test = (i % volume[dim+1]) + dir*volume[dim];

				if(test < 0)
					npos += volume[dim+1];
				else if(test >= volume[dim+1])
					npos -= volume[dim+1];
				
				neighbors[2*D*i + c] = npos;
				c++;
			}
		}
	}

	// calculate energy (Potts model)
	int E = 0;
	for(unsigned i=0; i<N; i++)	
	{
		for(unsigned j=0; j<2*D; j++)
		{
			if(spins[i] == spins[neighbors[2*D*i + j]])
				E--;
		}
	}
	E /= 2; // count each interaction only once


	std::cout<<"CPU L="<<L<<"\n";

	long long sum_E = 0;

	for(float T=0.4; T<=2.63; T+=0.05)
	{
		sum_E = 0;

		// run Metropolis algorithm
		for(unsigned i=0; i<N_SWEEPS; i++)
		{
			for(unsigned j=0; j<N; j++)
			{
				// Metropolis:
				short spinstate = spins[j];	// spin state

				// propose random new spin state:
				unsigned newstate = floor(drand48() * Q);

				// energy difference: E'-E
				int E_before = 0;
				int E_after  = 0;

				for(unsigned short n=0; n<2*D; n++)
				{
					if(spinstate == spins[neighbors[2*D*j + n]])
						E_before++;
					
					if(newstate == spins[neighbors[2*D*j + n]])
						E_after++;
				}

				float dE = static_cast<float>(E_before - E_after);
				float pAccept = exp(-dE/T);

				if(drand48() <= pAccept)
				{
					spins[j] = newstate;   		// flip spin
					E += E_before - E_after;	// note energy change
				}
			}

			if(i > 0.2*N_SWEEPS)	// sum up energies for mean value
				sum_E += E;
		}

		// mean energy:
//		double mE_Ising = 2 * static_cast<double>(sum_E) / (0.8*N_SWEEPS) / static_cast<double>(N) + 2;

//		std::cout<<T<<"\t"<<1.0f/(2.0*T)<<"\t"<<mE_Ising<<"\t"<<-log10(-mE_Ising)<<"\n";
	}

	return 0;
}